POSTGRESQL = "postgresql"
BIGQUERY = "bigquery"
SNOWFLAKE = "snowflake"
REDSHIFT = "redshift"


REDSHIFT_RESERVED_KEYWORDS = ["time", "timestamp"]
BIGQUERY_RESERVED_KEYWORDS = []
SNOWFLAKE_RESERVED_KEYWORDS = []

NUMERICAL_DATA_TYPES = [
    "smallint",
    "integer",
    "bigint",
    "decimal",
    "decimal",
    "numeric",
    "real",
    "double precision",
    "smallserial",
    "serial",
    "bigserial",
]
